!
! Supplementary Material, Software, for the Textbook:
!
! Mudelsee M (2020) Statistical Analysis of Climate Extremes,
!                   Cambridge University Press, 200 pp,
!                   ISBN 978-1-107-03318-4 (Hardback),
!                   ISBN 978-1-108-79146-5 (Paperback),
!                   https://www.manfredmudelsee.com/textbook
!
!=======================================================================
!
! AMICE: Action Measure for the Impact of Climate Extremes
!
!=======================================================================
!
!
! Copyright notice:     Copyright (C)
!                       Climate Risk Analysis - Manfred Mudelsee
!                       https://www.climate-risk-analysis.com
!
! Permission notice:    Permission is hereby granted, free of charge,
!                       to any person obtaining a copy of this software
!                       and associated computer files -- excluding the
!                       Numerical Recipes file(s) -- (the "Software"),
!                       to deal in the Software without restriction,
!                       including without limitation the rights to use,
!                       copy, modify, merge, publish, distribute,
!                       sublicense, and/or sell copies of the Software,
!                       and to permit persons to whom the Software is
!                       furnished to do so, subject to the following
!                       conditions:
!
!                       The above copyright notice, including the URL
!                       https://www.climate-risk-analysis.com,
!                       and this permission notice shall be included
!                       in all copies or substantial portions of the
!                       Software.
!
!=======================================================================
!
! Disclaimer:           THE SOFTWARE IS PROVIDED "AS IS", WITHOUT
!                       WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
!                       INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
!                       MERCHANTABILITY, FITNESS FOR A PARTICULAR
!                       PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
!                       THE AUTHOR OR COPYRIGHT HOLDER BE LIABLE FOR ANY
!                       CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
!                       ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
!                       FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
!                       OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
!
!=======================================================================
!
! Developer:            Climate Risk Analysis - Manfred Mudelsee
!                       Kreuzstrasse 27
!                       Heckenbeck
!                       37581 Bad Gandersheim
!                       Germany
!
! Contact:              http://www.climate-risk-analysis.com
!                       mudelsee@climate-risk-analysis.com
!
!=======================================================================
!
! Fortran 90 Compilation
! ======================
!
! gfortran compiler command: gfortran -o AMICE.exe -O2 AMICE.f90
!
! Change Log
! ==========
!
! Version 1.0       March 2019
!
! Contact
! =======
!
! Manfred Mudelsee
! Climate Risk Analysis
! Kreuzstrasse 27
! Heckenbeck
! 37581 Bad Gandersheim
! Germany
! Phone +49 5563 999 81 40
! Email mudelsee@climate-risk-analysis.com
! URL http://www.climate-risk-analysis.com
!
! ======================================================================
!
! Numerical Recipes File
! ======================
!
include 'nrtype.f90'
!
program amice
      use nrtype
      implicit none
!
      character(len=1)   :: answ1 = '-'           ! answer
      character(len=1)   :: flago = '-'           ! flag (output)
      character(len=1)   :: flags = '-'           ! flag for SGS
      character(len=5)   :: thrtype = '-----'     ! threshold type
                                                  ! thrtype = 'upper'
                                                  !           'lower'
      character(len=80)  :: infile                ! input data file
      character(len=20)  :: outfile1              ! output file, part 1
      character(len=7)   :: outfile2              ! output file, part 2
      character(len=240) :: outfilefull           ! output file
                                                  ! (full version)
      character(len=240) :: outfileshort          ! output file
                                                  ! (short version)
      character(len=240) :: outfilendays          ! output file
                                                  ! (number of days)
      character(len=240) :: outfilecsdi           ! output file
                                                  ! (CSDI)
      character(len=240) :: outfilewsdi           ! output file
                                                  ! (WSDI)
      character(len=240) :: outfilesgs            ! output file
                                                  ! (SGS)
      character(len=240) :: outfilelfd            ! output file
                                                  ! (LFD)
!
      integer :: n = -999                         ! data size
      integer, parameter :: nmax = 100000         ! maximum data size
                                                  ! (can be adapted to
                                                  !  computer system)
      integer,  dimension(nmax) :: t  = -999      ! time (counter)
      integer,  dimension(nmax) :: d  = -999      ! day
      integer,  dimension(nmax) :: m  = -999      ! month
      integer,  dimension(nmax) :: yr = -999      ! year
      real(sp), dimension(nmax) :: x  = -999.0_sp ! data value
!
      integer :: i                                ! counter
      integer :: idays                            ! number of days
                                                  ! measure
      integer :: dur                              ! duration
      integer :: durmin                           ! minimum duration
      integer, dimension(1) :: midi = -999        ! data point number
                                                  ! nearest to mid
      integer :: yrold=-999                       ! old year
      integer :: c=-999                           ! CSDI
      integer :: cold=-999                        ! old CSDI
      integer :: f=-999                           ! LFD
      integer :: fold=-999                        ! old LFD
      integer, parameter :: fsummerpeak=183       ! summer-peak day
      integer :: s=-999                           ! SGS
      integer :: smin=32                          ! minimum SGS 1 Feb
      integer :: sold=-999                        ! old SGS
      integer :: w=-999                           ! WSDI
      integer :: wold=-999                        ! old WSDI
      real(sp) :: thres = -999.0_sp               ! threshold value
      real(sp) :: mid = -999.0_sp                 ! midpoint time
      real(sp) :: a = -999.0_sp                   ! action measure
!
! 1.  Initial stuff
!     =============
!
      print *
      print *
      print *
      print *,                                                          &
       ' AMICE: Action Measure for the Impact of Climate Extremes'
      print *,                                                          &
       ' ========================================================'
      print *
      print *
      print *,                                                          &
       ' Mudelsee M (2020) Statistical Analysis of Climate Extremes.'
      print *,                                                          &
       ' Cambridge University Press.'
      print *
      print *,                                                          &
       ' http://www.climate-risk-analysis.com'
      print *
      print *
      print *
      print *,' Note (1): Input data format'
      print *,' ==========================='
      print *
      print *,' Column 1:     t  time (counter)'
      print *,'        2:     d  day (1-31)'
      print *,'        3:     m  month (1-12)'
      print *,'        4:     yr year'
      print *,'        5:     x  data value'
      print *
      print *,' Assumptions:  - daily data (i.e., t-units = days)'
      print *,'               - t increases strictly monotonically'
      print *,'               - relevant seasons already extracted'
      print *,'               - no missing values'
      print *
      print *,' See AMICE.f90 for example data'
!
!     Example (Tmax, Potsdam, Germany, JJA season, interval 1893-2018):
!
!                    t      d m yr   x
!
!     [line 1]       152    1 6 1893 15.4 
!     [line 2]       153    2 6 1893 17.5 
!     [line 3]       154    3 6 1893 21.8 
!     [line 4]       155    4 6 1893 23.3 
!      ...              
!     [line 29]      180   29 6 1893 24.8 
!     [line 30]      181   30 6 1893 22.9 
!     [line 31]      182    1 7 1893 17.2 
!     [line 32]      183    2 7 1893 24.1 
!      ...              
!     [line 91]      242   30 8 1893 18.8
!     [line 92]      243   31 8 1893 16
!     [line 93]      517    1 6 1894 19.7
!     [line 94]      518    2 6 1894 18.1
!      ...
!     [line 11589]   45895 28 8 2018 23
!     [line 11590]   45896 29 8 2018 28.1
!     [line 11591]   45897 30 8 2018 20.1
!     [line 11592]   45898 31 8 2018 21
!
      print *
      print *
      print *
      print *,' Note (2): Threshold selection'
      print *,' ============================='
      print *
      print *,' Assumption:   - constant threshold '
      print *
      print *,'               - application of time-dependent threshold'
      print *,'                 can be done prior to analysis'
      print *
      print *
      print *
      print *,' Note (3): Output files'
      print *,' ======================'
      print *
      print *,                                                          &
      ' AMICE-*.dat        - Action measure, full information'
      print *
      print *,                                                          &
      ' AMICEshort-*.dat   - Action measure, short version, no header'
      print *
      print *,' AMICEndays-*.dat   - Number of days measure'
      print *
      print *,' AMICE-CSDI-*.dat   - CSDI'
      print *
      print *,' AMICE-WSDI-*.dat   - WSDI'
      print *
      print *,' AMICE-SGS--*.dat   - Start of growing season'
      print *
      print *,' AMICE-LFD--*.dat   - Last frost day'
!
      print *
      print *
      print *
      print *,' Input data file name:'
      read (5,*) infile
      open (unit=1, file=infile, form='formatted', status='old')
      print *
      print *
      print *
      print '(a,i7,a)', ' Input data size (maximum: ',nmax,'):'
      read (5,*) n
      do i=1,n
         read (1,*) t(i),d(i),m(i),yr(i),x(i)
      end do
      close (unit=1, status='keep')
!
! test
!      do i=1,n
!         print *, t(i),d(i),m(i),yr(i),x(i)
!         if (mod(i,20) .eq. 0) read(*,*)
!      end do
!
!
! 2.  Input of calculation parameters
!     ===============================
!
      print *
      print *
      print *
      print *,' x-threshold, type:   upper [u]'
      print *,'                      lower [l]'
      read (5,*) answ1
      if (answ1 .eq. 'u') then
         thrtype='upper'
      else if (answ1 .eq. 'l') then
         thrtype='lower'
      end if
      print *
      print *
      print *
      print *,' x-threshold, value:        [INPUT]'
      read (5,*) thres
      print *
      print *
      print *
      print *,' Event, minimum duration:   [INPUT]'
      read (5,*) durmin
!
! 3.  Calculation of measure and output
!     =================================
!
!
! 3.1 Output files
!     ============
!
      open  (unit=1, status='scratch', form='formatted',                &
             access='sequential', action='readwrite')
      write  (unit=1, fmt='(f0.1)') thres
      rewind (unit=1)
      read   (unit=1, fmt=*) outfile1
      rewind (unit=1)
      write  (unit=1, fmt='(i0)') durmin
      rewind (unit=1)
      read   (unit=1, fmt=*) outfile2
      close  (unit=1)
!
      outfilefull  = 'AMICE-'//trim(adjustl(infile))//                  &
                     '-'//thrtype//'-'//                                &
                     trim(adjustl(outfile1))//'-durmin-'//              &
                     trim(adjustl(outfile2))//'.dat'
      outfileshort = 'AMICEshort-'//trim(adjustl(infile))//             &
                     '-'//thrtype//'-'//                                &
                     trim(adjustl(outfile1))//'-durmin-'//              &
                     trim(adjustl(outfile2))//'.dat'
      outfilendays = 'AMICEndays-'//trim(adjustl(infile))//             &
                     '-'//thrtype//'-'//                                &
                     trim(adjustl(outfile1))//'-durmin-'//              &
                     trim(adjustl(outfile2))//'.dat'
      outfilecsdi  = 'AMICE-CSDI-'//trim(adjustl(infile))//             &
                     '-'//thrtype//'-'//                                &
                     trim(adjustl(outfile1))//'-durmin-'//              &
                     trim(adjustl(outfile2))//'.dat'
      outfilewsdi  = 'AMICE-WSDI-'//trim(adjustl(infile))//             &
                     '-'//thrtype//'-'//                                &
                     trim(adjustl(outfile1))//'-durmin-'//              &
                     trim(adjustl(outfile2))//'.dat'
      outfilesgs   = 'AMICE-SGS--'//trim(adjustl(infile))//             &
                     '-'//thrtype//'-'//                                &
                     trim(adjustl(outfile1))//'-durmin-'//              &
                     trim(adjustl(outfile2))//'.dat'
      outfilelfd   = 'AMICE-LFD--'//trim(adjustl(infile))//             &
                     '-'//thrtype//'-'//                                &
                     trim(adjustl(outfile1))//'-durmin-'//              &
                     trim(adjustl(outfile2))//'.dat'
!
      open (unit=1, file=outfilefull, form='formatted',                 &
            status='unknown')
      open (unit=2, file=outfileshort, form='formatted',                &
            status='unknown')
      open (unit=3, file=outfilendays, form='formatted',                &
            status='unknown')
      if (thrtype .eq. 'upper') then
         open (unit=4, file=outfilewsdi, form='formatted',              &
               status='unknown')
         open (unit=8, file=outfilesgs,  form='formatted',              &
               status='unknown')
      else if (thrtype .eq. 'lower') then
         open (unit=7, file=outfilecsdi, form='formatted',              &
               status='unknown')
         open (unit=9, file=outfilelfd,  form='formatted',              &
               status='unknown')
      end if
      write (unit=1, fmt='(a)')                                         &
       ' AMICE: Results'
      write (unit=1, fmt='(a)')                                         &
       ' =============='
      write (unit=1, fmt='(a)')                                         &
       ' '
      write (unit=1, fmt='(a,a)')                                       &
       ' Input data file: ',trim(adjustl(infile))
      write (unit=1, fmt='(a)')                                         &
       ' '
      write (unit=1, fmt='(a,a)')                                       &
       ' Threshold type:  ',thrtype
      write (unit=1, fmt='(a)')                                         &
       ' '
      write (unit=1, fmt='(a,f0.1)')                                    &
       ' Threshold value: ',thres
      write (unit=1, fmt='(a)')                                         &
       ' '
      write (unit=1, fmt='(a,i0)')                                      &
       ' Event, minimum duration: ',durmin
      write (unit=1, fmt='(a)')                                         &
       ' '
      write (unit=1, fmt='(3(a))')                                      &
       ' Midpoint_time__d__m___yr_(rounded_up)',                        &
       ' Duration',                                                     &
       ' Action_measure'
!
      write (unit=3, fmt='(a)')                                         &
       ' AMICE: Results'
      write (unit=3, fmt='(a)')                                         &
       ' =============='
      write (unit=3, fmt='(a)')                                         &
       ' '
      write (unit=3, fmt='(a,a)')                                       &
       ' Input data file: ',trim(adjustl(infile))
      write (unit=3, fmt='(a)')                                         &
       ' '
      write (unit=3, fmt='(a,a)')                                       &
       ' Threshold type:  ',thrtype
      write (unit=3, fmt='(a)')                                         &
       ' '
      write (unit=3, fmt='(a,f0.1)')                                    &
       ' Threshold value: ',thres
      write (unit=3, fmt='(a)')                                         &
       ' '
      write (unit=3, fmt='(a,i0)')                                      &
       ' Event, minimum duration: ',durmin
      write (unit=3, fmt='(a)')                                         &
       ' '
      write (unit=3, fmt='(2(a))')                                      &
       ' yr  ',                                                         &
       ' #days_measure'
!
      if (thrtype .eq. 'upper') then
         write (unit=4, fmt='(a)')                                      &
         ' AMICE: Results'
         write (unit=4, fmt='(a)')                                      &
          ' =============='
         write (unit=4, fmt='(a)')                                      &
          ' '
         write (unit=4, fmt='(a,a)')                                    &
          ' Input data file: ',trim(adjustl(infile))
         write (unit=4, fmt='(a)')                                      &
          ' '
         write (unit=4, fmt='(a,a)')                                    &
          ' Threshold type:  ',thrtype
         write (unit=4, fmt='(a)')                                      &
          ' '
         write (unit=4, fmt='(a,f0.1)')                                 &
          ' Threshold value: ',thres
         write (unit=4, fmt='(a)')                                      &
          ' '
         write (unit=4, fmt='(a,i0)')                                   &
          ' Event, minimum duration: ',durmin
         write (unit=4, fmt='(a)')                                      &
          ' '
         write (unit=4, fmt='(2(a))')                                   &
          ' yr  ',                                                      &
          ' WSDI'
!
          write (unit=8, fmt='(a)')                                     &
         ' AMICE: Results'
         write (unit=8, fmt='(a)')                                      &
          ' =============='
         write (unit=8, fmt='(a)')                                      &
          ' '
         write (unit=8, fmt='(a,a)')                                    &
          ' Input data file: ',trim(adjustl(infile))
         write (unit=8, fmt='(a)')                                      &
          ' '
         write (unit=8, fmt='(a,a)')                                    &
          ' Threshold type:  ',thrtype
         write (unit=8, fmt='(a)')                                      &
          ' '
         write (unit=8, fmt='(a,f0.1)')                                 &
          ' Threshold value: ',thres
         write (unit=8, fmt='(a)')                                      &
          ' '
         write (unit=8, fmt='(a,i0)')                                   &
          ' Event, minimum duration: ',durmin
         write (unit=8, fmt='(a)')                                      &
          ' '
         write (unit=8, fmt='(2(a))')                                   &
          ' yr  ',                                                      &
          ' Start-of-growing-season'
      else if (thrtype .eq. 'lower') then
         write (unit=7, fmt='(a)')                                      &
          ' AMICE: Results'
         write (unit=7, fmt='(a)')                                      &
          ' =============='
         write (unit=7, fmt='(a)')                                      &
          ' '
         write (unit=7, fmt='(a,a)')                                    &
          ' Input data file: ',trim(adjustl(infile))
         write (unit=7, fmt='(a)')                                      &
          ' '
         write (unit=7, fmt='(a,a)')                                    &
          ' Threshold type:  ',thrtype
         write (unit=7, fmt='(a)')                                      &
          ' '
         write (unit=7, fmt='(a,f0.1)')                                 &
          ' Threshold value: ',thres
         write (unit=7, fmt='(a)')                                      &
          ' '
         write (unit=7, fmt='(a,i0)')                                   &
          ' Event, minimum duration: ',durmin
         write (unit=7, fmt='(a)')                                      &
          ' '
         write (unit=7, fmt='(2(a))')                                   &
          ' yr  ',                                                      &
          ' CSDI'
!
          write (unit=9, fmt='(a)')                                     &
         ' AMICE: Results'
         write (unit=9, fmt='(a)')                                      &
          ' =============='
         write (unit=9, fmt='(a)')                                      &
          ' '
         write (unit=9, fmt='(a,a)')                                    &
          ' Input data file: ',trim(adjustl(infile))
         write (unit=9, fmt='(a)')                                      &
          ' '
         write (unit=9, fmt='(a,a)')                                    &
          ' Threshold type:  ',thrtype
         write (unit=9, fmt='(a)')                                      &
          ' '
         write (unit=9, fmt='(a,f0.1)')                                 &
          ' Threshold value: ',thres
         write (unit=9, fmt='(a)')                                      &
          ' '
         write (unit=9, fmt='(a,i0)')                                   &
          ' Event, minimum duration: ',durmin
         write (unit=9, fmt='(a)')                                      &
          ' '
         write (unit=9, fmt='(2(a))')                                   &
          ' yr  ',                                                      &
          ' Last-frost-day'
       end if
!
! 3.2 Calculation and output
!     ======================
!
!     Note (1): For the action measure,
!               one special case of seasonal extremes
!               has not been treated:
!               threshold exceedances at the end of a season
!               and also at the beginning of the following season.
!               For example, if for JJA seasons the threshold is
!               exceeded on:
!               29 08 2003, 30 08 2003, 31 08 2003 and then on
!                1  6 2004,  2  6 2004, then
!               this would count as one event of duration 5 days with a
!               midpoint somewhere in mid-January 2004.
!               This case can be corrected for by means of an analysis
!               of the output file.
!
!     Note (2): For the number of days measure,
!               the CSDI, or the WSDI---
!               and a season that extends into the new year,
!               the simplest solution consists in an
!               adaptation of the input file years.
!
! 3.2.1  Action measure, upper threshold exceedances
!        ===========================================
!
      if (thrtype .eq. 'upper') then
         flago='n'                    ! flag (output)
         dur=0                        ! duration
         mid=0.0                      ! midpoint
         midi=0                       ! data point number nearest to mid
         a=0.0                        ! action measure
lcou:    do i=1,n
            if (x(i) .gt. thres) then ! upper
               dur=dur+1
               mid=(1.0*t(i)+mid*(dur-1.0))/real(dur)
               midi=minloc(abs(t(1:n)-NINT(mid)))
               a=a+x(i)-thres         ! action = exceedances added
                                      ! a > 0: upper threshold events
                                      ! a < 0: lower threshold events
               flago='y'
               if (i .eq. n .and. dur .ge. durmin) then
                  write (unit=1, fmt=                                   &
                   '(3x,f11.3,1x,i2,1x,i2,1x,i4,15x,i7,1x,f14.3)')      &
                   mid, d(midi(1)),                                     &
                        m(midi(1)),                                     &
                       yr(midi(1)), dur, a
                  write (unit=2, fmt=                                   &
                   '(1x,f11.3,1x,f14.3)')                               &
                   mid, a
                  exit lcou
               end if
            else                      ! output and re-initialize
               if (flago .eq. 'y' .and. dur .ge. durmin) then
                  write (unit=1, fmt=                                   &
                   '(3x,f11.3,1x,i2,1x,i2,1x,i4,15x,i7,1x,f14.3)')      &
                   mid, d(midi(1)),                                     &
                        m(midi(1)),                                     &
                       yr(midi(1)), dur, a
                  write (unit=2, fmt=                                   &
                   '(1x,f11.3,1x,f14.3)')                               &
                   mid, a
               end if
               flago='n'
               dur=0
               mid=0.0
               a=0.0
            end if
         end do lcou
!
! 3.2.2  Action measure, lower threshold exceedances
!        ===========================================
!
      else if (thrtype .eq. 'lower') then
         flago='n'                    ! flag (output)
         dur=0                        ! duration
         mid=0.0                      ! midpoint
         midi=0                       ! data point number nearest to mid
         a=0.0                        ! action measure
lcol:    do i=1,n
            if (x(i) .lt. thres) then ! lower
               dur=dur+1
               mid=(1.0*t(i)+mid*(dur-1.0))/real(dur)
               midi=minloc(abs(t(1:n)-NINT(mid)))
               a=a+x(i)-thres         ! action = exceedances added
                                      ! a > 0: upper threshold events
                                      ! a < 0: lower threshold events
               flago='y'
               if (i .eq. n .and. dur .ge. durmin) then
                  write (unit=1, fmt=                                   &
                   '(3x,f11.3,1x,i2,1x,i2,1x,i4,15x,i7,1x,f14.3)')      &
                   mid, d(midi(1)),                                     &
                        m(midi(1)),                                     &
                       yr(midi(1)), dur, a
                  write (unit=2, fmt=                                   &
                   '(1x,f11.3,1x,f14.3)')                               &
                   mid, a
                  exit lcol
               end if
            else                      ! output and re-initialize
               if (flago .eq. 'y' .and. dur .ge. durmin) then
                  write (unit=1, fmt=                                   &
                   '(3x,f11.3,1x,i2,1x,i2,1x,i4,15x,i7,1x,f14.3)')      &
                   mid, d(midi(1)),                                     &
                        m(midi(1)),                                     &
                       yr(midi(1)), dur, a
                  write (unit=2, fmt=                                   &
                   '(1x,f11.3,1x,f14.3)')                               &
                   mid, a
               end if
               flago='n'
               dur=0
               mid=0.0
               a=0.0
            end if
         end do lcol
      end if
!
! 3.2.3  Number of days measure, upper threshold exceedances
!        ===================================================
!
      if (thrtype .eq. 'upper') then
         idays=0                      ! number of days measure
         yrold=yr(1)
lciu:    do i=1,n
            if (i .eq. n) then
               write (unit=3, fmt=                                      &
               '(1x,i4,1x,i3)')                                         &
               yrold,count(yr .eq. yrold .and. x .gt. thres)
               exit lciu
            end if
            if (yr(i) .gt. yrold) then
               write (unit=3, fmt=                                      &
               '(1x,i4,1x,i3)')                                         &
               yrold,count(yr .eq. yrold .and. x .gt. thres)
               yrold=yr(i)
            end if
         end do lciu
      end if
!
! 3.2.4  Number of days measure, lower threshold exceedances
!        ===================================================
!
      if (thrtype .eq. 'lower') then
         idays=0                      ! number of days measure
         yrold=yr(1)
lcil:    do i=1,n
            if (i .eq. n) then
               write (unit=3, fmt=                                      &
               '(1x,i4,1x,i3)')                                         &
               yrold,count(yr .eq. yrold .and. x .lt. thres)
               exit lcil
            end if
            if (yr(i) .gt. yrold) then
               write (unit=3, fmt=                                      &
               '(1x,i4,1x,i3)')                                         &
               yrold,count(yr .eq. yrold .and. x .lt. thres)
               yrold=yr(i)
            end if
         end do lcil
      end if
!
! 3.2.5  WSDI (upper threshold exceedances)
!        ==================================
!
      if (thrtype .eq. 'upper') then
         yrold=-999                   ! old year
         dur=0                        ! duration
         w=0                          ! WSDI
         wold=0                       ! old WSDI
lcwu:    do i=1,n
            if (yr(i) .gt. yrold .and. yrold .ne. -999) then
               write (unit=4, fmt=                                      &
                   '(1x,i4,1x,i3)')                                     &
                   yrold,w
               dur=0
               w=0
               wold=0
            end if
            if (i .eq. n) then
               write (unit=4, fmt=                                      &
                   '(1x,i4,1x,i3)')                                     &
                   yrold,w
               exit lcwu
            end if
!
! Note: Since there are > 1 data points per season and year, the two
!       conditions: (yr(i) .gt. yrold .and. yrold .ne. -999),
!                   (i .eq. n)
!       are in practice not met simultaneously, and, hence,
!       double output is in practice prevented.
!
            if (x(i) .gt. thres) then ! upper
               dur=dur+1
               if (dur .ge. durmin) w=wold+dur
            else                      ! re-initialize
               dur=0
               wold=w
            end if
            yrold=yr(i)
         end do lcwu
      end if
!
! 3.2.6  CSDI (lower threshold exceedances)
!        ==================================
!
      if (thrtype .eq. 'lower') then
         yrold=-999                   ! old year
         dur=0                        ! duration
         c=0                          ! CSDI
         cold=0                       ! old CSDI
lccl:    do i=1,n
            if (yr(i) .gt. yrold .and. yrold .ne. -999) then
               write (unit=7, fmt=                                      &
                   '(1x,i4,1x,i3)')                                     &
                   yrold,c
               dur=0
               c=0
            end if
            if (i .eq. n) then
               write (unit=7, fmt=                                      &
                   '(1x,i4,1x,i3)')                                     &
                   yrold,c
               exit lccl
            end if
            if (x(i) .lt. thres) then ! lower
               dur=dur+1
               if (dur .ge. durmin) c=cold+dur
            else                      ! re-initialize
               dur=0
               cold=c
            end if
            yrold=yr(i)
         end do lccl
      end if
!
! 3.2.7  Start of growing season, first upper threshold exceedance
!        =========================================================
!
      if (thrtype .eq. 'upper') then
         yrold=-999                   ! old year
         dur=0                        ! duration
         s=0                          ! SGS
         sold=0                       ! old SGS
         flags='n'                    ! flag for SGS
lcog:    do i=1,n
            if (yr(i) .gt. yrold .and. yrold .ne. -999) then
               write (unit=8, fmt=                                      &
                   '(1x,i4,1x,i3)')                                     &
                   yrold,sold
               dur=0
               s=0
               flags='n'
            end if
            if (i .eq. n) then
               write (unit=8, fmt=                                      &
                   '(1x,i4,1x,i3)')                                     &
                   yrold,sold
               exit lcog
            end if
!
! Note: Since there are > 1 data points per season and year, the two
!       conditions: (yr(i) .gt. yrold .and. yrold .ne. -999),
!                   (i .eq. n)
!       are in practice not met simultaneously, and, hence,
!       double output is in practice prevented.
!
            if (x(i) .gt. thres) then ! upper
               dur=dur+1
               if (dur .ge. durmin .and. flags .eq. 'n'                 &
                                   .and. s .ge. (smin-1)) then
                  sold=s+1
                  flags='y'
               end if
            else                      ! re-initialize
               dur=0
            end if
            s=s+1
            yrold=yr(i)
         end do lcog
      end if
!
! 3.2.8  Last frost day, last lower threshold exceedance
!        ===============================================
!
      if (thrtype .eq. 'lower') then
         idays=0                             ! counter of days in year
         f=-999                              ! last frost day in year
                                             ! before summer peak
         yrold=-999
lcof:    do i=1,n
            idays=idays+1
            if (i .eq. n) then
               write (unit=9, fmt=                                      &
               '(1x,i4,1x,i3)')                                         &
               yrold,f
               exit lcof
            end if
            if (yr(i) .gt. yrold .and. yrold .ne. -999) then
               write (unit=9, fmt=                                      &
               '(1x,i4,1x,i3)')                                         &
               yrold,f
               yrold=yr(i)
               idays=1
               f=-999
            end if
            if (idays .lt. fsummerpeak .and. x(i) .lt. thres) then
               f=idays
               yrold=yr(i)
            end if
         end do lcof
      end if
!
! 4.  Exit
!     ======
!
      close (unit=1, status='keep')
      close (unit=2, status='keep')
      close (unit=3, status='keep')
      if (thrtype .eq. 'upper') then
         close (unit=4, status='keep')
         close (unit=8, status='keep')
      else if (thrtype .eq. 'lower') then
         close (unit=7, status='keep')
         close (unit=9, status='keep')
      end if
      print *
      print *
      print *
      print *, ' AMICE terminated successfully'
      print *, ' ============================='
      print *
      stop
end program amice